/*
 * HeinzelJPanel.java
 *
 * Created on January 20, 2007, 4:30 PM
 */

package info.heinzelnisse.he;

import java.awt.Dimension;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.StringSelection;
import java.awt.datatransfer.Transferable;
import java.awt.event.KeyEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.jnlp.ClipboardService;
import javax.jnlp.ServiceManager;
import javax.jnlp.UnavailableServiceException;
import javax.swing.JViewport;
import javax.swing.SwingUtilities;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import javax.swing.text.Utilities;
import javax.swing.text.html.HTMLEditorKit;

/**
 *
 * @author  Heiko Klein
 */
public class HeinzelJPanel extends javax.swing.JPanel implements PropertyChangeListener, SearchKeyEventListener {
    static private Logger logger = Logger.getLogger(HeinzelJPanel.class.getName());
        
    private SearchWindow searchWindow;
    
    /** Creates new form HeinzelJPanel */
    public HeinzelJPanel() {
        initComponents();
        ControllerBean controller = ControllerBean.getInstance();
        controller.addPropertyChangeListener(this);
        controller.addSearchKeyEventListener(this);
        jCheckBox2.setSelected(controller.isNorwegian());
        jCheckBox1.setSelected(controller.isGerman());
        indexPanel1.setController(controller);
        searchPanel1.setController(controller);
    }

    /**
     * exposes the slider location of the main jSplitPane1
     */
    public int getDividerLocation() {
        return jSplitPane1.getDividerLocation();
    }
    /**
     * sets the slider location of the main jSplitPane1
     */
    public void setDividerLocation(int location) {
        jSplitPane1.setDividerLocation(location);
    }

    private void copyClipboard(String word) {
        if (ControllerBean.getInstance().isCutnpaste()) {
            try {
                ClipboardService clipboardService = (ClipboardService) ServiceManager.lookup("javax.jnlp.ClipboardService");
                clipboardService.setContents(new StringSelection(word));
            } catch (UnavailableServiceException ex) {
                Logger.getLogger(HeinzelJFrame.class.getName()).info("ClipboardSercice: " + ex.toString());
                Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
                clipboard.setContents(new StringSelection(word), null);
            }
        }
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jSplitPane1 = new javax.swing.JSplitPane();
        jPanel1 = new javax.swing.JPanel();
        searchPanel1 = new info.heinzelnisse.he.SearchPanel();
        indexPanel1 = new info.heinzelnisse.he.IndexPanel();
        jPanel2 = new javax.swing.JPanel();
        jPanel3 = new javax.swing.JPanel();
        jCheckBox1 = new javax.swing.JCheckBox();
        jCheckBox2 = new javax.swing.JCheckBox();
        jButton1 = new javax.swing.JButton();
        jButton2 = new javax.swing.JButton();
        jScrollPane1 = new javax.swing.JScrollPane();
        jEditorPane1 = new javax.swing.JEditorPane();

        searchPanel1.setBorder(javax.swing.BorderFactory.createEtchedBorder());
        searchPanel1.setMinimumSize(new java.awt.Dimension(90, 42));
        searchPanel1.setPreferredSize(new java.awt.Dimension(95, 42));

        indexPanel1.setBorder(javax.swing.BorderFactory.createEtchedBorder());

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(indexPanel1, javax.swing.GroupLayout.DEFAULT_SIZE, 90, Short.MAX_VALUE)
            .addComponent(searchPanel1, javax.swing.GroupLayout.DEFAULT_SIZE, 90, Short.MAX_VALUE)
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addComponent(searchPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(indexPanel1, javax.swing.GroupLayout.DEFAULT_SIZE, 89, Short.MAX_VALUE))
        );

        jSplitPane1.setLeftComponent(jPanel1);

        jPanel3.setBorder(javax.swing.BorderFactory.createEtchedBorder());

        jCheckBox1.setFont(new java.awt.Font("Dialog", 1, 10));
        jCheckBox1.setSelected(true);
        java.util.ResourceBundle bundle = java.util.ResourceBundle.getBundle("info/heinzelnisse/he/properties"); // NOI18N
        jCheckBox1.setText(bundle.getString("German")); // NOI18N
        jCheckBox1.setActionCommand("german");
        jCheckBox1.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        jCheckBox1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jCheckBox1ActionPerformed(evt);
            }
        });

        jCheckBox2.setFont(new java.awt.Font("Dialog", 1, 10));
        jCheckBox2.setSelected(true);
        jCheckBox2.setText(bundle.getString("Norwegian")); // NOI18N
        jCheckBox2.setActionCommand("norwegian");
        jCheckBox2.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        jCheckBox2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jCheckBox2ActionPerformed(evt);
            }
        });

        jButton1.setIcon(new javax.swing.ImageIcon(getClass().getResource("/info/heinzelnisse/he/resources/left.png"))); // NOI18N
        jButton1.setToolTipText(bundle.getString("backward")); // NOI18N
        jButton1.setEnabled(false);
        jButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton1ActionPerformed(evt);
            }
        });

        jButton2.setIcon(new javax.swing.ImageIcon(getClass().getResource("/info/heinzelnisse/he/resources/right.png"))); // NOI18N
        jButton2.setToolTipText(bundle.getString("forward")); // NOI18N
        jButton2.setEnabled(false);
        jButton2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton2ActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout jPanel3Layout = new javax.swing.GroupLayout(jPanel3);
        jPanel3.setLayout(jPanel3Layout);
        jPanel3Layout.setHorizontalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel3Layout.createSequentialGroup()
                .addComponent(jButton1, javax.swing.GroupLayout.PREFERRED_SIZE, 15, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jButton2, javax.swing.GroupLayout.PREFERRED_SIZE, 15, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 14, Short.MAX_VALUE)
                .addComponent(jCheckBox1, javax.swing.GroupLayout.PREFERRED_SIZE, 79, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jCheckBox2)
                .addContainerGap())
        );
        jPanel3Layout.setVerticalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel3Layout.createSequentialGroup()
                .addGroup(jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jButton2, javax.swing.GroupLayout.PREFERRED_SIZE, 13, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jButton1, javax.swing.GroupLayout.PREFERRED_SIZE, 13, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addGroup(jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(jCheckBox2, javax.swing.GroupLayout.PREFERRED_SIZE, 15, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addComponent(jCheckBox1)))
                .addGap(19, 19, 19))
        );

        jEditorPane1.setEditable(false);
        jEditorPane1.setEditorKit(new HTMLEditorKit());
        jEditorPane1.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                jEditorPane1MouseClicked(evt);
            }
        });
        jScrollPane1.setViewportView(jEditorPane1);

        javax.swing.GroupLayout jPanel2Layout = new javax.swing.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jPanel3, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
            .addComponent(jScrollPane1, javax.swing.GroupLayout.Alignment.TRAILING)
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addComponent(jPanel3, javax.swing.GroupLayout.PREFERRED_SIZE, 21, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 110, Short.MAX_VALUE))
        );

        jSplitPane1.setRightComponent(jPanel2);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jSplitPane1, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 334, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jSplitPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 137, Short.MAX_VALUE)
        );
    }// </editor-fold>//GEN-END:initComponents
    
    private void jButton2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton2ActionPerformed
        ControllerBean.getInstance().getHistoryNext();
    }//GEN-LAST:event_jButton2ActionPerformed
    
    private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed
        ControllerBean.getInstance().getHistoryPrevious();
    }//GEN-LAST:event_jButton1ActionPerformed
    
    private void jCheckBox2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jCheckBox2ActionPerformed
        ControllerBean.getInstance().setNorwegian(jCheckBox2.isSelected());
    }//GEN-LAST:event_jCheckBox2ActionPerformed
    
    private void jCheckBox1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jCheckBox1ActionPerformed
        ControllerBean.getInstance().setGerman(jCheckBox1.isSelected());
    }//GEN-LAST:event_jCheckBox1ActionPerformed

    private void jEditorPane1MouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jEditorPane1MouseClicked
        try {
            Point p = evt.getPoint();
            int pos = jEditorPane1.viewToModel(p);
            int start = Utilities.getWordStart(jEditorPane1, pos);
            int end = Utilities.getWordEnd(jEditorPane1, pos);
            String word = jEditorPane1.getText(start, end-start);
            switch (evt.getButton()) {
                case java.awt.event.MouseEvent.BUTTON1: ControllerBean.getInstance().setSearchString(word); break;
                case java.awt.event.MouseEvent.BUTTON3: copyClipboard(word); break;
                default: ;
            }
        } catch (BadLocationException ex) {
            Logger.getLogger(HeinzelJPanel.class.getName()).log(Level.SEVERE, null, ex);
        }
    }//GEN-LAST:event_jEditorPane1MouseClicked
    
    public void propertyChange(PropertyChangeEvent pce) {
        final PropertyChangeEvent evt = pce;
        logger.fine("getting event: "+evt.getPropertyName() + " " + evt.getNewValue());
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                if (evt.getPropertyName().equals(ControllerBean.NORWEGIAN)) {
                    jCheckBox2.setSelected((Boolean) evt.getNewValue());
                    indexPanel1.fillJList1();
                } else if (evt.getPropertyName().equals(ControllerBean.GERMAN)) {
                    jCheckBox1.setSelected((Boolean) evt.getNewValue());
                    indexPanel1.fillJList1();
                } else if (evt.getPropertyName().equals(ControllerBean.TRANSLATION)) {
                    updateJEditorPane1();
                } else if (evt.getPropertyName().equals(ControllerBean.HISTORY_NEXT)) {
                    jButton2.setEnabled((Boolean)evt.getNewValue());
                } else if (evt.getPropertyName().equals(ControllerBean.HISTORY_PREVIOUS)) {
                    jButton1.setEnabled((Boolean)evt.getNewValue());
                }
            }
        });
    }
    
    private void updateJEditorPane1() {
        TranslationGroup tg = ControllerBean.getInstance().getTranslationGroup();
        StringBuilder sb = new StringBuilder();
        sb.append("<html><body>");
        (new HTMLTranslationFormatter()).format(sb, tg);
        sb.append("</body></html>");
        final String text = sb.toString();
        final javax.swing.JEditorPane jEditorPane1 = this.jEditorPane1;
        SwingUtilities.invokeLater(new Runnable() {
            @Override
            public void run() {
                Document doc = jEditorPane1.getDocument();
                jEditorPane1.setCaretPosition(0);
                try {
                    doc.remove(0, doc.getLength());
                } catch (BadLocationException ex) {
                    ex.printStackTrace(); // cannot happen
                }
                jEditorPane1.setText(text);
                // go to start of text
                jEditorPane1.moveCaretPosition(0);
                jEditorPane1.select(0,0);
            }
        });
    }
    
    public void searchKeyReleased(KeyEvent evt) {
        logger.info("got key event"+ evt);
        switch (evt.getKeyCode()) {
            case KeyEvent.VK_PAGE_UP: scrollPage(true) ; break;
            case KeyEvent.VK_PAGE_DOWN: scrollPage(false); break;
            default:
        }
    }
    
    private void scrollPage(boolean up) {
        logger.info("scrolling "+up);
        JViewport vp = jScrollPane1.getViewport();
        Point p = vp.getViewPosition();
        Dimension d = vp.getExtentSize();
        p.translate(0, (int) ((up ? -.8 : 1.6) * d.getHeight()));
        jEditorPane1.setCaretPosition(jEditorPane1.viewToModel(p));
        
    }
    
    void openSearchWindow() {
        if (searchWindow == null) {
            searchWindow = new SearchWindow();
            searchWindow.setController(ControllerBean.getInstance());
        }
        searchWindow.setText(ControllerBean.getInstance().getSearchString());
        searchWindow.setState(java.awt.Frame.NORMAL);
        searchWindow.setVisible(true);
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private info.heinzelnisse.he.IndexPanel indexPanel1;
    private javax.swing.JButton jButton1;
    private javax.swing.JButton jButton2;
    private javax.swing.JCheckBox jCheckBox1;
    private javax.swing.JCheckBox jCheckBox2;
    private javax.swing.JEditorPane jEditorPane1;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JSplitPane jSplitPane1;
    private info.heinzelnisse.he.SearchPanel searchPanel1;
    // End of variables declaration//GEN-END:variables
}
