package info.heinzelnisse.he;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.logging.Logger;

/**
 * Title:        HeinzelSearch
 * Description:  Desktop Edition of Heinzelnisse
 * Copyright:    Copyright (c) 2003
 * Company:
 * @author Heiko Klein
 * @version 0.1
 */

public class HeinzelSearch {
    final static String EACH = "?";
    final static String ANY = "*";
    private static Logger logger = Logger.getLogger(HeinzelSearch.class.getName());

    public static List<String> items(String str, List<String> items) {
        String searchStr = searchString(str);
        String preStr = preString(searchStr);
        int idx;
        if ((str.charAt(0) == ANY.charAt(0))
            || (str.charAt(0) == EACH.charAt(0))) {
            idx = 0;
        } else {
            idx = Collections.binarySearch(items, preStr, HeinzelTable.getDeNoCollator());
        }
        Iterator<String> it;
        if (idx < 0) {
            it = items.listIterator(-1 * idx - 1);
        } else {
            it = items.listIterator(idx);
        }
        List<String> out = new ArrayList<String>();
        while (it.hasNext()) {
            String next = it.next();
            int compareResults = compareItems(searchStr.toLowerCase(Locale.GERMAN), next.toLowerCase(Locale.GERMAN));
            if (compareResults == -1) {
                break;
            } else if (compareResults == 1) {
                out.add(next);
            }
        }
        return out;
    }

    private static String preString(String str) {
        int idx = str.indexOf(ANY);
        if (idx < 0)
            idx = str.length();
        int idx2 = str.indexOf(EACH);
        if (idx2 < 0)
            idx2 = str.length();
        idx = (idx < idx2) ? idx : idx2;
        return str.substring(0, idx);
    }

    private static String searchString(String str) {
        return str.trim();
    }

    // return 1 on equal
    //        0 on not equal, but expecting more
    //       -1 on not equal, no more
    private static int compareItems(String str, String next) {
        String preStr = preString(str);
        int minLength =
            (preStr.length() < next.length()) ? preStr.length() : next.length();
        if (minLength != 0) {
            if (!preStr
                .substring(0, minLength)
                .equals(next.substring(0, minLength)))
                return -1;
        }

        if (compareStartFrom(0, str.toCharArray(), 0, next.toCharArray())) {
            return 1;
        } else {
            return 0;
        }
    }

    private static boolean compareAnyStart(
        int regpos,
        char[] regex,
        int otherpos,
        char[] other) {
        int thisotherpos = otherpos;

        while ((regpos < regex.length) && (regex[regpos] == ANY.charAt(0))) {
            regpos++;
        }
        if (regpos >= regex.length)
            return true;
        if (thisotherpos >= other.length)
            return false;

        while (!compareChars(regex[regpos], other[thisotherpos])) {
            thisotherpos++;
            if (thisotherpos >= other.length)
                return false;
        }

        if (compareStartFrom(regpos, regex, thisotherpos, other)) {
            return true;
        } else {
            return compareAnyStart(regpos, regex, (thisotherpos + 1), other);
        }
    }

    private static boolean compareStartFrom(
        int regpos,
        char[] regex,
        int otherpos,
        char[] other) {
        int thisregpos = regpos;
        int thisotherpos = otherpos;

        if (other.length == 0)
            return false;
        if (regex.length == 0)
            return false;

        while (compareChars(regex[thisregpos], other[thisotherpos])) {
            thisregpos++;
            thisotherpos++;
            if ((thisregpos == regex.length) || (thisotherpos == other.length))
                break;
        }

        if (thisregpos < regex.length) {
            if (regex[thisregpos] == ANY.charAt(0))
                return compareAnyStart(
                    (thisregpos + 1),
                    regex,
                    thisotherpos,
                    other);
        }

        if ((thisregpos == regex.length) && (thisotherpos == other.length)) {
            return true;
        } else {
            return false;
        }
    }

    private static boolean compareChars(char a, char b) {
        if (a == EACH.charAt(0))
            return true;
        return (a == b);
    }
}
